==> tokio-interruptible-future-5.0.0.crate: tokio-interruptible-future-5.0.0/src/lib.rs <==
/// Easily interrupt async code in given check points. It's useful to interrupt threads/fibers.
/// TODO: Documentation comments.

use std::{fmt, future::Future};
use std::sync::Arc;
use tokio::sync::broadcast::error::{RecvError, SendError, TryRecvError};
use tokio::sync::Mutex;

#[derive(Debug, PartialEq, Eq)]
pub struct InterruptError { }

impl InterruptError {
    pub fn new() -> Self {
        Self { }
    }
}

impl fmt::Display for InterruptError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "Async fiber interrupted.")
    }
}

/// tokio::sync::broadcast::Sender has `Receiver` not cloneable, creating the temptation to
/// clone an `Arc` with `Receiver` inside, what would lead to loss of messages.
///
/// So, this instead (make it a separate library?)
pub fn broadcast<'a, T: Clone>(capacity: usize) -> (Sender<T>, Receiver<T>) {
    let (tx, rx) = tokio::sync::broadcast::channel(capacity);
    let tx = Arc::new(Mutex::new(tx));
    return (
        Sender {
            tx: tx.clone(),
        },
        Receiver {
            tx: tx.clone(),
            rx
        }
    )
}

pub struct Sender<T> {
    tx: Arc<Mutex<tokio::sync::broadcast::Sender<T>>>,
}

pub struct Receiver<T> {
    tx: Arc<Mutex<tokio::sync::broadcast::Sender<T>>>,
    rx: tokio::sync::broadcast::Receiver<T>,
}

impl<T> Sender<T> {
    pub async fn receiver_count(&self) -> usize {
        self.tx.lock().await.receiver_count()
    }
    pub async fn send(&self, value: T) -> Result<usize, SendError<T>> {
        self.tx.lock().await.send(value)
    }
}

impl<T: Clone> Receiver<T> {
    pub async fn recv(&mut self) -> Result<T, RecvError> {
        self.rx.recv().await
    }
    pub fn try_recv(&mut self) -> Result<T, TryRecvError> {
        self.rx.try_recv()
    }
    pub async fn receiver_count(&self) -> usize {
        self.tx.lock().await.receiver_count()
    }
    pub async fn clone(&self) -> Self {
        Self {
            tx: self.tx.clone(),
            rx: self.tx.lock().await.subscribe(),
        }
    }
}

/// You usually use `interruptible` or `interruptible_sendable` instead.
pub async fn interruptible_straight<T, E: From<InterruptError>>(
    rx: Receiver<()>,
    f: impl Future<Output=Result<T, E>>
) -> Result<T, E>
{
    let mut rx = rx;
    tokio::select!{
        r = f => r,
        _ = async { // shorten lock lifetime
            let _ = rx.recv().await;
        } => Err(InterruptError::new().into()),
    }
}

pub async fn interruptible<T, E: From<InterruptError>>(
    rx: Receiver<()>,
    f: impl Future<Output=Result<T, E>> + Unpin
) -> Result<T, E>
{
    interruptible_straight(rx, f).await
}

pub async fn interruptible_sendable<T, E: From<InterruptError>>(
    rx: Receiver<()>,
    f: impl Future<Output=Result<T, E>> + Send + Unpin
) -> Result<T, E>
{
    interruptible_straight(rx, f).await
}

/// TODO: More tests.
#[cfg(test)]
mod tests {
    use std::future::Future;
    use futures::executor::block_on;

    use crate::{InterruptError, interruptible, interruptible_sendable, broadcast};

    #[derive(Debug, PartialEq, Eq)]
    struct AnotherError { }
    impl AnotherError {
        pub fn new() -> Self {
            return Self { }
        }
    }
    #[derive(Debug, PartialEq, Eq)]
    enum MyError {
        Interrupted(InterruptError),
        Another(AnotherError)
    }
    impl From<InterruptError> for MyError {
        fn from(value: InterruptError) -> Self {
            Self::Interrupted(value)
        }
    }
    impl From<AnotherError> for MyError {
        fn from(value: AnotherError) -> Self {
            Self::Another(value)
        }
    }
    struct Test {
    }
    impl Test {
        pub fn new() -> Self {
            Self {
            }
        }
        pub async fn g(self) -> Result<u8, MyError> {
            let (_tx, rx) = broadcast::<()>(1);

            interruptible(rx, Box::pin(async move {
                Ok(123)
            })).await
        }
        pub async fn h(self) -> Result<u8, MyError> {
            let (_tx, rx) = broadcast::<()>(1);

            interruptible(rx, Box::pin(async move {
                Err(AnotherError::new().into())
            })).await
        }
    }

    #[test]
    fn interrupted() {
        let test = Test::new();
        block_on(async {
            assert_eq!(test.g().await, Ok(123));
        });
        let test = Test::new();
        block_on(async {
            assert_eq!(test.h().await, Err(AnotherError::new().into()));
        });
    }

    #[test]
    fn check_interruptible_sendable() {
        let (_tx, rx) = broadcast::<()>(1);

        // Check that `interruptible_sendable(...)` is a `Send` future.
        let _: &(dyn Future<Output = Result<i32, InterruptError>> + Send) = &interruptible_sendable(rx, Box::pin(async move {
            Ok(123)
        }));
    }
}

==> tokio-interruptible-future-6.0.0.crate: tokio-interruptible-future-6.0.0/src/lib.rs <==
/// Easily interrupt async code in given check points. It's useful to interrupt threads/fibers.
/// TODO: Documentation comments.

use std::{fmt, future::Future};
use tokio::sync::futures::Notified;

#[derive(Debug, PartialEq, Eq)]
pub struct InterruptError { }

impl InterruptError {
    pub fn new() -> Self {
        Self { }
    }
}

impl fmt::Display for InterruptError {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "Async fiber interrupted.")
    }
}

/// You usually use `interruptible` or `interruptible_sendable` instead.
pub async fn interruptible_straight<'a, T, E: From<InterruptError>>(
    notified: Notified<'a>,
    f: impl Future<Output=Result<T, E>>
) -> Result<T, E>
{
    tokio::select!{
        r = f => r,
        _ = async { // shorten lock lifetime
            let _ = notified.await;
        } => Err(InterruptError::new().into()),
    }
}

pub async fn interruptible<'a, T, E: From<InterruptError>>(
    notified: Notified<'a>,
    f: impl Future<Output=Result<T, E>> + Unpin
) -> Result<T, E>
{
    interruptible_straight(notified, f).await
}

pub async fn interruptible_sendable<'a, T, E: From<InterruptError>>(
    notified: Notified<'a>,
    f: impl Future<Output=Result<T, E>> + Send + Unpin
) -> Result<T, E>
{
    interruptible_straight(notified, f).await
}

/// TODO: More tests.
#[cfg(test)]
mod tests {
    use std::future::Future;
    use futures::executor::block_on;
    use tokio::sync::Notify;

    use crate::{InterruptError, interruptible, interruptible_sendable};

    #[derive(Debug, PartialEq, Eq)]
    struct AnotherError { }
    impl AnotherError {
        pub fn new() -> Self {
            return Self { }
        }
    }
    #[derive(Debug, PartialEq, Eq)]
    enum MyError {
        Interrupted(InterruptError),
        Another(AnotherError)
    }
    impl From<InterruptError> for MyError {
        fn from(value: InterruptError) -> Self {
            Self::Interrupted(value)
        }
    }
    impl From<AnotherError> for MyError {
        fn from(value: AnotherError) -> Self {
            Self::Another(value)
        }
    }
    struct Test {
    }
    impl Test {
        pub fn new() -> Self {
            Self {
            }
        }
        pub async fn g(self) -> Result<u8, MyError> {
            let notify = Notify::new();
            let notified = notify.notified();

            interruptible(notified, Box::pin(async move {
                Ok(123)
            })).await
        }
        pub async fn h(self) -> Result<u8, MyError> {
            let notify = Notify::new();
            let notified = notify.notified();

            interruptible(notified, Box::pin(async move {
                Err(AnotherError::new().into())
            })).await
        }
    }

    #[test]
    fn interrupted() {
        let test = Test::new();
        block_on(async {
            assert_eq!(test.g().await, Ok(123));
        });
        let test = Test::new();
        block_on(async {
            assert_eq!(test.h().await, Err(AnotherError::new().into()));
        });
    }

    #[test]
    fn check_interruptible_sendable() {
        let notify = Notify::new();
        let notified = notify.notified();

        // Check that `interruptible_sendable(...)` is a `Send` future.
        let _: &(dyn Future<Output = Result<i32, InterruptError>> + Send) = &interruptible_sendable(notified, Box::pin(async move {
            Ok(123)
        }));
    }
}
